/********************************************************************************************************
Macro Name:   fitRE.sas
Author:       Matthew Psioda
Affiliation:  Department of Biostatistics, University of North Carolina at Chapel Hill
    
**** Program Summary:
      
    This program uses PROC MCMC to fit a recurrent events model separately to each study in libname1.dataset1 along with 
    incorporating historical data through use of the power prior, if applicable.

**** Macro Variables Needed;
    (1) method          = <SLICE|NORMAL||t(6)> determines the type of sampler, specify a SAS supported proposal distribution for MH
    (2) inDS            = <libname1.dataset1> dataset that contains the study datasets in the format generated by the simRE.sas macro
    (3) mcmcOptions     = <text> valid SAS code for PROC MCMC line [See Note 1]
    (4) outDS           = <libname2.dataset2|libname3.dataset3> dataset to store summarized PROC MCMC output
    (5) mcmcSeed        = <integer|NULL> seed for PROC MCMC, does not set seed if NULL

    (6) frailtyModel    = <1|0> 1 = fit gamma frailty model; 0 = fit non-homogeneous PPM 
    (7) sValues         = <double|double|...> change points for baseline hazard for new study    

    (8) a0              = <double|NONE> control parameter for power prior information borrowing [See Note 2]
        (8.1) hc_sValues         = <double|double|...> change points for baseline hazard for historical study
        (8.2) hc_loglambda_start = <double|double|...> starting values for MCMC sampler
        (8.3) hc_logTauInv_start = <double> starting value for MCMC sampler
        (8.4) hcDataset          = <double> historical control dataset	
    (9) phi0List        = <double|...> list of phi0 values used to determine rejection rule
    (10) delta          = superiority margin
	(11) odsOptions    = <sas_expr> options to control ods output to listing/html destinations;
    Note 1:
        *** intended for nmc, nbi, nthin, ntu (or other MH tuning options);
        *** example 1 : ntu=500 nbi=500 nmc=50000 nthin=2

    Note 2:
        *** NONE indicates no power prior design
        *** (8.x) variables only need by set if a0 is not NONE

**** Output Summary;

     (A) libname2.dataset2 - sas dataset containing summarized MCMC output
        (1) Parameter           - (text) name of parameter
        (2) PostProb            - (double) value of P(exp(gamma)<delta | Data)
        (3) numSamples          - (integer) number of MCMC samples kept
        (4) mean                - (double) posterior mean
        (5) median              - (double) posterior median
        (6) effectiveNumSamples - (double) effective number of samples (measure of sampler performance)
        (7) gewekeStatistic     - (double) Geweke Statistic for associated parameter
        (8) gewekeProbZ         - (double) p-value for Geweke Statistic for associated parameter

     (B) libname3.dataset3 - sas dataset containing the summarized power/type I error rates
        (1) RejRatej - Rejection rate for j^th element of Phi0List macro variable
      
*********************************************************************************************************/


%macro fitRE(
            method             = NORMAL,
            inDS               = work.simStudy,
			mcmcOptions        = ntu=2000 mintune=10 nbi=1000 nmc=120000 nthin=8 propcov=Quanew plots=(none),
			outDS              = work.mcmcResults|work.mcmcRejRates,
			mcmcSeed           = NULL,
			frailtyModel       = 0,
			sValues            = 10|20|30,
			a0                 = NONE,
			    hc_sValues         = ,
			    hc_loglambda_start = ,
			    hc_logTauInv_start = ,
			    hcDataset          = ,
			phi0List           = 0.95|0.96,
			delta              = 0.975,
			odsOptions         = ods listing close; ods html close;
			);
			
			
%put %upcase(no)TE: *********************** Macro Variable Values ***********************;
%put   ;
%put %upcase(no)TE: method             = &method.;
%put %upcase(no)TE: inDS               = &inDS.;
%put %upcase(no)TE: mcmcOptions        = &mcmcOptions.;
%put %upcase(no)TE: outDS              = &outDS.;
%put %upcase(no)TE: mcmcSeed           = &mcmcSeed.;
%put %upcase(no)TE: frailtyModel       = &frailtyModel.;
%put %upcase(no)TE: sValues            = &sValues.;
%put %upcase(no)TE: a0                 = &a0.;
%put %upcase(no)TE: 	hc_sValues         = &hc_sValues.;
%put %upcase(no)TE: 	hc_loglambda_start = &hc_loglambda_start.;
%put %upcase(no)TE: 	hc_logTauInv_start = &hc_logTauInv_start.;
%put %upcase(no)TE: 	hcDataset          = &hcDataset.;
%put %upcase(no)TE: phi0List           = &phi0List.;
%put %upcase(no)TE: delta              = &delta.;
%put %upcase(no)TE: odsOptions         = &odsOptions.;

			
			
			
Options MERGENOBY= NOWARN;

%if &a0 ^= NONE %then %do;
 %blendHC;
 %let _ds_ = _TEMP_;
 %let hc_K = %eval(%sysfunc(count(&hc_sValues.,|))+2);
%end;
%else %do;
 %let _ds_ = &inDS.;
%end;


data _null_;
 call symput('startTime',strip(time()));
run;

data _null_;
 set &_ds_.(obs=1);
  array x(*) x:;
  call symput('p',strip(put(dim(x),best.)));
  
  array nInt(*) nInt:;
  call symput('maxK',strip(put(dim(nInt),best.)));
 run;
%let K    = %eval(%sysfunc(count(&sValues.,|))+2);


sasfile &_ds_. load;

&odsOptions;
ods output PostSummaries = PostSummaries Geweke=Geweke ESS=ESS;

proc mcmc data = &_ds_. &mcmcOptions. %if &mcmcSeed. ^= NULL %then %do; seed = &mcmcSeed. %end; 
  STATISTICS(PERCENTAGE=50)=SUMMARY DIAGNOSTICS=(ESS GEWEKE)
   monitor=( gammaHalfSpace gamma
             %do i = 1 %to &p.; beta&i. %end;
             %do i = 1 %to &K.; lambda&i. %end;
             %do i = 1 %to &K.; loglambda&i. %end; %if &frailtyModel = 1 %then %do; tau logtauInv %end; 

             %if &a0 ^= NONE %then %do; 
               %do i = 1 %to &hc_K.; hc_lambda&i. %end;
               %do i = 1 %to &hc_K.; hc_loglambda&i. %end; %if &frailtyModel = 1 %then %do; hc_tau hc_logtauInv %end; 
             %end;

            );
 by studyReplicate;
 
    BEGINCNST ; 
        K0 = &K.;
        array s(%eval(&K+1)) (0 %do i = 1 %to %eval(&K.-1); %scan(&sValues,&i,|) %end; 100000000);
        %if &a0 ^= NONE %then %do;  
         hc_K0 = &hc_K.;
         array hc_s(%eval(&hc_K.+1)) (0 %do i = 1 %to %eval(&hc_K.-1); %scan(&hc_sValues.,&i,|) %end; 100000000); 
        %end;
    ENDCNST; 
     
     
        array nInt(&maxK.);
        array loglambda[&K.];
        array lambda[&K.];

        %if &a0 ^= NONE %then %do; 
          array hc_loglambda[&hc_K.];
          array hc_lambda[&hc_K.];
        %end;
          
        array logRate[2,2];
        array expLogRate[2,2];
          
        parms %do i = 1 %to &K.; loglambda&i. %sysfunc(log(0.15)) %end; 
              %if &frailtyModel = 1 %then %do; logtauInv -0.5 %end; / &method.;
        prior %do i = 1 %to &K.; loglambda&i. %end; ~ normal(0,var=1000);
        %if &frailtyModel = 1 %then %do; prior logtauInv ~ normal(0,var=1000);  %end; 


        %if &a0 ^= NONE %then %do; 
          parms %do i = 1 %to &hc_K.; hc_loglambda&i. %scan(&hc_loglambda_start.,&i,|) %end; 
                 %if &frailtyModel = 1 %then %do; hc_logtauInv &hc_logTauInv_start. %end; / &method.;
          prior %do i = 1 %to &hc_K.; hc_loglambda&i. %end; ~ normal(0,var=1000,lower=-8,upper=8);
          %if &frailtyModel = 1 %then %do; prior hc_logtauInv ~ normal(0,var=1000,lower=-6,upper=6); %end;
        %end;

        parms %do i = 1 %to &p; beta&i. 0 %end; 
              gamma 0 / &method.;
        prior %do i = 1 %to &p.; beta&i. %end; gamma ~ general(0);



     BEGINNODATA ; 
      %if &frailtyModel = 1 %then %do;
        tauInv        = exp(logtauInv);
        tau           = 1/tauInv;
        logTau        = -logtauInv;
        lgammaTauInv  = lgamma(tauInv);
      %end;

        do k = 1 to &K.; 
           lambda[k] = exp(loglambda[k]);
        end;  

        %if &a0 ^= NONE %then %do;    
          %if &frailtyModel = 1 %then %do;
            hc_tauInv = exp(hc_logtauInv);
            hc_tau    = 1/hc_tauInv;
            hc_logTau = -hc_logtauInv;
            hc_lgammaTauInv = lgamma(hc_tauInv);
          %end;
          do k = 1 to &hc_K.; 
             hc_lambda[k] = exp(hc_loglambda[k]);
          end;
        %end;
 
        gammaHalfSpace = (exp(gamma)<&Delta.);

          do _z_ = 0 to 1;
          do _x1_ = 0 to 1;
            logRate[_z_+1,_x1_+1]    = _z_*gamma + _x1_*beta1;
            expLogRate[_z_+1,_x1_+1] = exp(logRate[_z_+1,_x1_+1]);
          end;
          end;
    ENDNODATA ; 
     
  %if &a0 ^= NONE %then %do;      
    if study = 'SIMULATION' then do;
       %logLikeRE(px=);
    end;
    else do;
       %logLikeRE(px=hc_,scale=&a0.);
    end;
  %end;
  %else %do;
       %logLikeRE(px=);
  %end;
  
    model general(logLike);
run;
quit;


sasfile &_ds_. close;


data _null_;
 call symput('ElapsedTime',strip( ( time()-&StartTime. )/(60)));
run;
%put %upcase(no)TE: Bayesian Model Fitting Elapsed Time (minutes) = &ElapsedTime.;


data %scan(&outDS.,1,|); 
 length Parameter $30.;
 retain studyReplicate Parameter PostProb ;
 merge PostSummaries ESS Geweke;

  if Parameter = 'gammaHalfSpace' then do;
    Parameter = 'P(exp(gamma)<delta | Data)';
    ESS = .;
    zScore = .;
    pValue = .;
    P50 = .;
    PostProb = mean;
    Mean=.;
    N=.;
  end;
  else do;
   PostProb = .;
  end;

  keep studyReplicate Parameter PostProb  N mean P50 ESS zScore pValue;

  rename N      = numSamples 
         P50    = median
         ESS    = effectiveNumSamples
         zScore = gewekeStatistic
         pValue = gewekeProbZ;
run;

%let numPhi0 = %eval(%sysfunc(count(&phi0List.,|))+1);
data temp;
 set %scan(&outDS.,1,|);
  where Parameter = 'P(exp(gamma)<delta | Data)';
  array Rej(&numPhi0.);
  
 %do j = 1 %to &numPhi0.;
   if PostProb > %scan(&phi0List.,&j.,|) then Rej(&j.) = 1; else Rej(&j.) = 0;
 %end;
run;

proc univariate data = temp noprint;
 var %do j = 1 %to &numPhi0.; Rej&j. %end;;
 output out = %scan(&outDS.,2,|) mean =  %do j = 1 %to &numPhi0.; RejRate&j. %end;;
run;

data %scan(&outDS.,2,|);
 set %scan(&outDS.,2,|);
 
    label 
       %do j = 1 %to &numPhi0.; 
        RejRate&j. = "Rejection Rate for phi0=%scan(&phi0List.,&j.,|)"
      %end;;
run;


proc datasets library=work noprint;
 delete PostSummaries ESS Geweke temp;
run;
quit;
Options MERGENOBY= WARN;
%mend;
