/********************************************************************************************************
Macro Name:   simRE.sas
Author:       Matthew Psioda
Affiliation:  Department of Biostatistics, University of North Carolina at Chapel Hill
    
**** Program Summary:
      
     This program generates data from a gamma frailty random effects model and collapses the data
     down to the number of events that each subject has during the period where each hazard component
     is applicable.

**** Macro Variables Required;


 *** Simulation Control Variables;
    (1) studyRepNum     = <integer> number of simulated studies to perform per loop
        simLoopCount    = <integer> number of simulation loops [See Note 3]
    (2) simSeedValues   = <integer|integer> seeds for random number generation, can use NULL|NULL to avoid setting seed
    (3) storeSim        = <libname1.dataset1> output dataset names for simulated data, must specify libnamex (even if work)

 *** Design Variables;
    (4) trialDuration   = <double> overall duration of the trial
    (5) n               = <integer> number of subjects to be enrolled
    (6) lambdaValues    = <double|double|...> K piece-wise constant hazard rates 
    (7) sValues         = <double|double|...> K-1 change points for baseline hazard
    (8) tauValue        = <double> parameter value for the gamma frailty variance
    (9) betaValue       = <double> parameter value for binary covariate
    (10) xDistGen       = <sas_expr> sas expression [See Note 1]
    (11) gammaValue     = <double> parameter value for treatment effect
    (12) zDistGen       = <sas_expr> [See Note 1]
    (13) CensDist       = <sas_expr> [See Notes 1 and 2]
     


    Note 1:
        *** must be valid in this context : x1 = sas_expr;
        *** example 1 : sas_expr = rand('Bernoulli',0.5);
    Note 2:
        *** example 1 : sas_expr = &trialDuration.; (censor at end of trial, no dropout);
        *** example 2 : sas_expr = round(rand('Uniform')*200+50);            
    Note 3: 
        *** It takes more time to generate data for a large number of studies at once than it does to
            generate data for a small number of studies many times (appending each time). These macro variables 
            allow the user to generate (studyRepNum X simLoopCount) studies and explore the best balance for their machine
               
**** Output Summary;

    (A) libname1.dataset1 - sas dataset containing n observations including the following variables
       (1) studyReplicate - (integer) for by processing in model fitting step (1 to studyRepNum X simLoopCount )
       (2) subjectID      - (integer) subject ID (1 to n)
       (3) z              - (integer) treatment covariates
       (4) x[j]           - (double) other covariates, [j]=1,...,p
       (5) censTime       - (double) time of censorship for the subject.       
       (6) nEvent         - (integer) overall number of events for the subject
       (7) nInt[j]        - (integer) number of events for the subject that occurred under hazard component [j], [j]=1,...K

*********************************************************************************************************/

%macro simRE(
            storeSim     = work.simStudy,
			studyRepNum  = 1,
			simLoopCount = 1,
            simSeedValues = NULL|NULL,
			
			trialDuration = 225,
			n             = 116,
            CensDist      = &trialDuration,
            lambdaValues  = 0.14|0.31|0.13|0.08, 	
            sValues       = 14|21|49,
            tauValue      =  0.000001,
            betaValue     = -1.92,
            gammaValue    = -0.17,
            xDistGen      = rand('bernoulli',0.44),
            zDistGen      = rand('bernoulli',0.50)
            );
			
%put %upcase(no)TE: *********************** Macro Variable Values ***********************;
%put   ;
%put %upcase(no)TE: *** Simulation Control Variables;
%put %upcase(no)TE:    (1.1) studyRepNum   = &studyRepNum.;
%put %upcase(no)TE:    (1.2) simLoopCount  = &simLoopCount.;
%put %upcase(no)TE:    (2) simSeedValues   = &simSeedValues.;
%put %upcase(no)TE:    (3) storeSim        = &storeSim.;
%put   ;
%put   ;
%put %upcase(no)TE: *** Design Variables;
%put %upcase(no)TE:    (4) trialDuration   = &trialDuration.;
%put %upcase(no)TE:    (5) n               = &n.;
%put %upcase(no)TE:    (6) lambdaValues    = &lambdaValues.;
%put %upcase(no)TE:    (7) sValues         = &sValues.;
%put %upcase(no)TE:    (8) tauValue        = &tauValue.;
%put %upcase(no)TE:    (9) betaValue       = &betaValue.;
%put %upcase(no)TE:    (10) xDistGen       = &xDistGen.;
%put %upcase(no)TE:    (11) gammaValue     = &gammaValue.;
%put %upcase(no)TE:    (12) zDistGen       = &zDistGen.;
%put %upcase(no)TE:    (13) CensDist       = &CensDist.;
			
			
Options MERGENOBY= NOWARN;
proc datasets library=%scan(%scan(&storeSim,1,|),1,.) noprint; 
   delete %scan(%scan(&storeSim,1,|),2,.); run; quit;


data _null_;
 call symput('startTime',strip(time()));
run;

 %let K = %eval(%sysfunc(count(&lambdaValues.,|))+1);
 %let p = %eval(%sysfunc(count(&betaValue.,|))+1);
 %let seed1 = %scan(&simSeedValues.,1,|);
 %let seed2 = %scan(&simSeedValues.,2,|);

 %do loop = 1 %to &simLoopCount.;

data Sample;
 %if &seed1. ^= NULL %then %do; call streaminit(&seed1.); %end; 


 *** Set design values;
 trialDur = &trialDuration.;
 n        = &n.;
 K        = &K.;

 array s(%eval(&K.+1)) %do i = 0 %to &K.; s&i. %end;
                     ( 0 %do i = 1 %to %eval(&K.-1); %scan(&sValues.,&i.,|) %end; 1000000 );

  *** Section 3.2 - Step 1;
 tau   = &tauValue.;

 %do i = 1 %to &K.;
  lambda&i. = %scan(&lambdaValues.,&i.,|);
 %end;

 gamma = &gammaValue.;

 %do cov = 1 %to &p.;
  beta&cov. = %scan(&betaValue.,&cov.,|);
 %end;

 do studyReplicate = 1 to &studyRepNum.;
 do subjectID = 1 to &N.;

  *** Section 3.2 - Step 2;
  z = &zDistGen.;

  %do cov = 1 %to &p.;
   x&cov. = %scan(&xDistGen.,&cov.,|);
  %end; 

  *** Section 3.2 - Step 3;
  if tau > 0 then omega = rand('gamma',1/tau)*tau;
  else omega = 1;

  *** Section 3.2 - Step 5 (censoring time);
  censTime = min(trialDur,&censDist.);

  output;
 end;
 end;

run;

proc IML;
 %if &seed2. ^= NULL %then %do; call streaminit(&seed2.); %end; 
 use Sample;

 *** read simulation level data;
 read point 1 var { %do i = 0 %to &K.; s&i. %end; } into s;
 read point 1 var { %do i = 1 %to &k.; lambda&i. %end; } into lambda;

 read point 1 var { %do cov = 1 %to &p.; beta&cov. %end; } into beta; beta=t(beta); 
 read point 1 var { gamma } into gamma; 
 read point 1 var {n} into n; n = n*&studyRepNum.;
 read point 1 var {K} into K;

 *** read subject level data;
 read all var { omega } into omega;
 read all var { z } into z;
 read all var { censTime } into censTime;
 read all var { %do cov = 1 %to &p.; x&cov. %end; } into x; 

 
 eventNumbers   = J(n,1,0);
 eventIntervals = J(n,K,0);

 do i = 1 to n;

  tEvent = 0; ** Current event time;
  nEvent = 0; ** Current event number;
  loop   = 0; ** Overall loop counter;


    expRate = lambda*omega[i]*exp(z[i]*gamma + x[i,]*beta);
    *print i expRate;

  do until(tEvent>=censTime);
    ** Determine applicable hazard for next event;
    hazPart = 1;
    do h = 1 to K;
     if tEvent > s[h] & tEvent <= s[h+1] then hazPart = h;
    end;

    ** Simulate event time;
    terminate = 0;
    do until(terminate=1);

      tAdd    = rand('exponential')/expRate[hazPart];
      tProp   = tEvent + tAdd;

      if hazPart < K & tProp > s[hazPart+1]  then do;
         tEvent  = s[hazPart+1];
         hazPart = hazPart+1;
      end;
      else do;
       terminate            = 1;
       tEvent               = tProp;
       keepEvent            = (tEvent<=censTime[i]);
       nEvent               = nEvent + keepEvent;
        if keepEvent = 1 then do;
          *eventTimes[i,nEvent] = tEvent;
          eventIntervals[i,hazPart] = eventIntervals[i,hazPart]+1;
        end;
      end;
    end;
  end;

  eventNumbers[i,1] = nEvent;
 end; 



 *maxEventNumber = max(eventNumbers);
 events = eventNumbers||eventIntervals;*||eventTimes[,1:maxEventNumber];

 clab1 = "nInt1":"nInt&K.";
 *clab2 = "t1":"t100000";
 clab = {"nEvent"}||clab1;*||clab2;

 create events from events[c=clab];
  append from events;
 close events;
run;
quit;

 data _temp_;
  retain studyReplicate subjectID z %do cov = 1 %to &p.; x&cov. %end;;
  merge Sample(drop=trialDur tau) Events;

  studyReplicate = studyReplicate + (&loop.-1)*&studyRepNum.;

  keep studyReplicate subjectID x: z censTime nEvent nInt: ;
 run; proc sort; by studyReplicate subjectID z x:; run;

 proc append data = _temp_ base = %scan(&storeSim,1,|) force; run; quit;
 proc datasets library=work noprint; delete Sample Events _temp_; run; quit;


%end;


data _null_;
 call symput('ElapsedTime',strip( ( time()-&StartTime. )/(60)));
run;



%put %upcase(no)TE: Simulated Data Generation Elapsed Time (minutes) = &ElapsedTime.;
Options MERGENOBY= WARN ;
%mend simRE;

